/**
 * chumby_inline_string.h - lightweight inline replacement std::string
 *
 * henry@chumby.com
 * This is intentionally built as an inline class which should not depend on templates or any library code.
 * It uses malloc, realloc and free
**/

#ifndef _FAKE_STRING_INCLUDED_
#define _FAKE_STRING_INCLDUED_

#include <stdio.h>
#include <malloc.h>
#include <string.h>

class cstring;

class cstring
{
public:
	cstring()
	{
		m_data = NULL;
		m_dataLength = 0;
	}
	cstring(const char *src)
	{
		m_data = NULL;
		m_dataLength = 0;
		*this = src;
	}
	cstring(const cstring& src)
	{
		m_data = NULL;
		m_dataLength = 0;
		*this = src;
	}
	cstring(const char *src, int length)
	{
		m_data = NULL;
		m_dataLength = 0;
		assign( src, length );
	}
	cstring(const cstring& src, int length)
	{
		m_data = NULL;
		m_dataLength = 0;
		assign( src.c_str(), length );
	}
	~cstring()
	{
		free_str();
	}
	const cstring& operator=( const cstring& src )
	{
		*this = src.c_str();
		return *this;
	}
	const cstring& operator=( const char *src )
	{
		free_str();
		m_dataLength = strlen(src);
		m_data = (char*)malloc( m_dataLength + 1 );
		if (m_data)
		{
			strcpy( m_data, src );
		}
		else
		{
			m_dataLength = 0;
		}
		return *this;
	}
	const cstring& assign( const cstring& src, int srcLength )
	{
		return assign( src.c_str(), srcLength );
	}
	const cstring& assign( const char *src, int srcLength )
	{
		free_str();
		m_dataLength = srcLength;
		m_data = (char*)malloc( m_dataLength + 1 );
		if (m_data)
		{
			strncpy( m_data, src, srcLength );
			m_data[srcLength] = '\0';
		}
		else
		{
			m_dataLength = 0;
		}
		return *this;
	}
	cstring operator+( const cstring& src ) const
	{
		cstring s(*this);
		s += src;
		return s;
	}
	cstring operator+( const char *src ) const
	{
		cstring s(*this);
		s += src;
		return s;
	}
	const cstring& operator+=( const cstring& src )
	{
		*this += src.c_str();
		return *this;
	}
	const cstring& operator+=( const char *src )
	{
		int orgDataLength = m_dataLength;
		m_dataLength += strlen(src);
		m_data = (char*)realloc( m_data, m_dataLength + 1 );
		strcpy(&m_data[orgDataLength], src);
		return *this;
	}
	const cstring& append( const cstring& src, int srcLength )
	{
		return append( src.c_str(), srcLength );
	}
	const cstring& append( const char *src, int srcLength )
	{
		int orgDataLength = m_dataLength;
		m_dataLength += srcLength;
		m_data = (char*)realloc( m_data, m_dataLength + 1 );
		strncpy( &m_data[orgDataLength], src, srcLength );
		m_data[m_dataLength] = '\0';
		return *this;
	}
	bool operator==( const char *tgt ) const
	{
		return (strcmp( tgt, m_data )==0);
	}
	bool operator!=( const char *tgt ) const
	{
		return (strcmp( tgt, m_data )!=0);
	}
	bool operator<(const char *tgt ) const
	{
		return (strcmp( m_data, tgt ) < 0);
	}
	bool operator>(const char *tgt ) const
	{
		return (strcmp( m_data, tgt ) > 0);
	}
	bool operator<=(const char *tgt ) const
	{
		return (strcmp( m_data, tgt ) <= 0);
	}
	bool operator>=(const char *tgt ) const
	{
		return (strcmp( m_data, tgt ) >= 0);
	}
	bool operator==( const cstring& tgt ) const
	{
		return (*this == tgt.c_str());
	}
	bool operator!=( const cstring& tgt ) const
	{
		return (*this != tgt.c_str());
	}
	bool operator<( const cstring& tgt ) const
	{
		return (*this < tgt.c_str());
	}
	bool operator>( const cstring& tgt ) const
	{
		return (*this > tgt.c_str());
	}
	bool operator<=( const cstring& tgt ) const
	{
		return (*this <= tgt.c_str());
	}
	bool operator>=( const cstring& tgt ) const
	{
		return (*this >= tgt.c_str());
	}
	operator const char *() const
	{
		return c_str();
	}
	const char *c_str() const
	{
		if (m_data)
			return m_data;
		return "";
	}
	int length() const { return m_dataLength; }

protected:
	void free_str()
	{
		if (m_data)
		{
			free(m_data);
			m_data = NULL;
			m_dataLength = 0;
		}
	}

	char *m_data;
	int m_dataLength;
};

#endif
