/**
 * mm_options.h - declaration of MMOptions class for parsing mountmon daemon options
 * Copyright (C) 2007 Chumby Industries. All Rights reserved
 * henry@chumby.com
*/
#ifndef _MM_OPTIONS_H_INCLUDED_
#define _MM_OPTIONS_H_INCLUDED_

#include "chumby_inline_string.h"

class MMOptions
{
public:
	MMOptions();
	~MMOptions();

	// Set configuration file. Default is /etc/mountmon/mountmon.conf
	void SetConf( const char *confPath );
	const char *GetConf() const { return m_conf[0].c_str(); }

	// Parse configuration. Return number of rules processed successfully.
	int ParseConfig( int verbose );

	// Get include depth count
	int GetIncludeDepth() const { return m_curInclude + 1; }

	// Get include at specified level, where 0 is the top level (first file to be parsed)
	const char *GetInclude( int depth ) const;

	// Get command paths
	const char *GetMountCmd() const { return m_mountCmd.c_str(); }
	const char *GetUmountCmd() const { return m_umountCmd.c_str(); }
	const char *GetAddCmd() const { return m_addCmd.c_str(); }
	const char *GetRemoveCmd() const { return m_removeCmd.c_str(); }

	// Get verbosity - -1 if not specified, 0 if off, 1 if on
	int GetVerbose() const { return m_verbose; }

	// Get environment variables set in the config
	int GetEnvCount() const { return m_envCount; }
	const char *GetEnvName( int index ) const { if (index>=0 && index<m_envCount) return m_envName[index].c_str(); else return ""; }
	const char *GetEnvValue( int index ) const { if (index>=0 && index<m_envCount) return m_envValue[index].c_str(); else return ""; }

protected:
#define MAX_INCLUDE_DEPTH 10
	cstring m_conf[MAX_INCLUDE_DEPTH];
	int m_curInclude;
	cstring m_mountCmd;
	cstring m_umountCmd;
	cstring m_addCmd;
	cstring m_removeCmd;
	int m_verbose; // Verbose parsed from config
	int m_verboseRequested; // Verbose passed into ParseConfig()
	int m_envCount;
#define MAX_ENV_CHANGES 32
	cstring m_envName[MAX_ENV_CHANGES];
	cstring m_envValue[MAX_ENV_CHANGES];

	int FindEnvName( const char *name );
	bool AddEnvDef( const char *name, const char *value );
	int ExpandEnvValues( cstring& s, int exclude );
};


#endif // _MM_OPTIONS_H_INCLUDED_
