#!/bin/sh
# $Id: add.sh 683 2007-06-07 02:39:21Z henry $
# add event script for mountmon
#
# See mountmon.conf for list of environment variables available on input
# and required output values
#

RESULT=0
MESSAGE=
MODULE_DIR=/drivers
DBGOUT=/dev/null
[ "${DAEMON_LOGDIR}" != "" ] && DBGOUT=${DAEMON_LOGDIR}/add.$$

SCRIPT_VER='$Rev: 683 $'
echo "# Starting $0 version ${SCRIPT_VER}"

# Get MMS_PUID and other useful stuff based on DEVPATH
. mountmon_utils.sh

# We need a special kluge for iPod, since we don't have the usual stuff
# passed on to us from udev and we need to roll our tortillas from scratch
if [ "${ID_VENDOR}" = "Apple" -a "${ID_MODEL}" = "iPod" -a "${DEVNAME}" != "" -a "${ID_FS_TYPE}" = "" ]
then
  # run volid
  eval "$(/lib/udev/vol_id ${DEVNAME} | grep -v ID_FS_LABEL=)"
fi

# At this point we've collected most of the vars that are of interest for debugging.
if [ "${MM_VERBOSE}" = "1" ]
then
	set >/tmp/mountmon.add.$$
fi

# For CD/DVD, another special case
if [ "${ID_FS_TYPE}" = "" -a "${DEVNAME}" != "" -a "${ID_TYPE}" = "cd" ]
then
  # run volid
  eval "$(/lib/udev/vol_id ${DEVNAME} | grep -v ID_FS_LABEL=)"
  # Determine module we need loaded
  NEED_MODULE=${ID_FS_TYPE}
  [ "${NEED_MODULE}" = "iso9660" ] && NEED_MODULE=isofs
  # Check to see if module is loaded
  LOAD_CHECK="$(lsmod | grep ${NEED_MODULE})"
  if [ "${LOAD_CHECK}" = "" ]
  then
  	echo "# kernel module ${NEED_MODULE} required, loading..."
  	if insmod ${MODULE_DIR}/${NEED_MODULE}.ko
  	then
  		echo "# module loaded successfully"
  		MESSAGE="${MESSAGE} ; loaded ${NEED_MODULE}"
  		sleep 1
  	else
  		RESULT=1
  		MESSAGE="Failed to load ${NEED_MODULE}"
  		ID_FS_TYPE=""
  	fi
  fi
fi

# More module loads triggered by specific vendor:modelid combos
# We put these in /etc/mountmon/*.moddef files to make it simpler
# to add new devices, but still handle matches individually here
if [ "${MMS_PUID}" != ":" ]
then

	# Check for chumbradiod
	if grep "^${MMS_PUID}" /etc/mountmon/chumbradiod.moddef >/dev/null
	then
		echo "# Starting chumbradiod for ${MMS_VENDOR_NAME} ${MMS_PRODUCT_NAME} ${MMS_VERSION}"
		MESSAGE="${MESSAGE} ; starting chumbradiod"
		service_control chumbradiod load ${DEVPATH} >>${DBGOUT} 2>&1
	fi

	# Check for USB ethernet adapters such as trendnet which use the pegasus driver
	if grep "^${MMS_PUID}" /etc/mountmon/pegasus.moddef >/dev/null
	then
		echo "# USB ethernet ${MMS_PUID}"
		loadKernelModule /drivers/usbnet
		loadKernelModule /drivers/pegasus
		# Determine the interface it's on
		ETH="$(ifconfig -a | awk '/^eth/ {print $1;}')"
		# If it's not visible wait
		if [ "${ETH}" = "" ]
		then
			echo "# Waiting for interface to become visible"
			sleep 2
			ETH="$(ifconfig -a | awk '/^eth/ {print $1;}')"
		fi
		# FIXME move this to start_network, which needs to handle possible manual config
		# Bring it up
		if [ "${ETH}" ]
		then
			# see if it's already up
			IFCONFIG_DATA="$(ifconfig ${ETH} | awk '/ RUNNING / {print;}' 2>/dev/null)"
			if [ "${IFCONFIG_DATA}" = "" ]
			then
				echo "# Starting network interface ${ETH}"
				ifconfig ${ETH} up
				# Start dhcp client if necessary
				DHCP_RUNNING=$(ps | awk "/[u]dhcpc .* ${ETH}/ {print 1;}")
				if [ "${DHCP_RUNNING}" != "1" ]
				then
					udhcpc -R -n -p /var/run/udhcpc.${ETH}.pid -i ${ETH}
				fi
			else
				echo "# ${ETH} is already running"
			fi
		else
			echo "# Warning - did not find interface in ifconfig -a, cannot start interface"
		fi
	fi

fi

# Mount filesystems
if [ "${DEVNAME}" -a "${ID_FS_TYPE}" ]
then

  # Make sure extra mount points exist
  for d in cdrom cdrom2 cdrom3 usb usb2 usb3 usb4 ipod ipod2 ipod3
  do
    [ -d /mnt/$d ] || mkdir /mnt/$d
  done

  # Do we need to find a mount point?
  MP=${MM_PREVIOUS_MOUNTPOINT}
  if [ "${MP}" != "" ]
  then
    echo "# using previous mount point"
  fi

  # Check for apple
  if [ "${MP}" = "" -a "${ID_VENDOR}" = "Apple" -a "${ID_MODEL}" = "iPod" ]
  then
  	# show mount points in preferred order
  	for testmp in /mnt/ipod*
  	do
  	  grep ${testmp} /proc/mounts >/dev/null 2>&1 || MP=${testmp}
  	  if [ "${MP}" ]
  	  then
  	    break
  	  fi
  	done
  fi

  # Check for CD/DVD
  if [ "${MP}" = "" -a "${ID_TYPE}" = "cd" ]
  then
  	for testmp in /mnt/cdrom*
  	do
  	  grep ${testmp} /proc/mounts >/dev/null 2>&1 || MP=${testmp}
  	  [ "${MP}" ] && break
  	done
  fi

  # Check for hard drives

  # Check for generic usb drives
  if [ "${MP}" = "" -a "${ID_BUS}" = "usb" ]
  then
  	echo "# default mount point"
  	for testmp in /mnt/usb*
  	do
  	  grep ${testmp} /proc/mounts >>${DBGOUT} 2>&1 || MP=${testmp}
  	  if [ "${MP}" ]
  	  then
  	    break
  	  fi
  	done
  fi

  # Mount if we have something
  if [ "${MP}" != "" ]
  then
  	echo "MOUNT=${MP}"
  	RESULT=1
  	mount ${DEVNAME} ${MP} -t ${ID_FS_TYPE} && MESSAGE="${MESSAGE}; Mounted ${DEVNAME} successfully" && RESULT=0
  else
  	MESSAGE="Unable to find a suitable free mount point for ${DEVNAME}"
  	RESULT=1
  fi

  # Display some additional info if verbose is on
  if [ "${RESULT}" = "0" -a "${ID_VENDOR}${ID_MODEL}" != "" ]
  then
    echo "# vendor/model/puid = ${ID_VENDOR}/${ID_MODEL}/${MMS_PUID}"
  fi

  # Check for daemons we need to start after a successful mount
  if [ "${ID_VENDOR}" = "Apple" -a "${ID_MODEL}" = "iPod" -a "${RESULT}" = "0" ]
  then
	echo "# Starting chumbipodd"
	MESSAGE="${MESSAGE}; starting chumbipodd"
	service_control chumbipodd load ${DEVPATH} >>${DBGOUT} 2>&1
  fi


fi

echo "RESULT=${RESULT}"
echo "MESSAGE=${MESSAGE}"

return ${RESULT}
